@@ -34,8 +34,7 @@ class ActiveRecord::ConnectionAdapters::NullDBAdapter < ActiveRecord::Connection
         self.class.const_get(config[:table_definition_class_name]))
     end
 
-    register_types unless NullDB::LEGACY_ACTIVERECORD || \
-                          ActiveRecord::VERSION::MAJOR < 4
+    register_types
   end
 
   # A log of every statement that has been "executed" by this connection adapter
@@ -74,44 +73,40 @@ class ActiveRecord::ConnectionAdapters::NullDBAdapter < ActiveRecord::Connection
 
     yield table_definition if block_given?
 
-    @tables[table_name] = table_definition
+    @tables[table_name.to_s] = table_definition
   end
 
-  def add_index(table_name, column_names, options = {})
+  def rename_table(table_name, new_name)
+    table_definition = @tables.delete(table_name.to_s)
+
+    table_definition.name = new_name.to_s
+    @tables[new_name.to_s] = table_definition
+  end
+
+  def add_index(table_name, column_names, **options)
+    options[:unique] = false unless options.key?(:unique)
     column_names = Array.wrap(column_names).map(&:to_s)
-    index_name, index_type, ignore = add_index_options(table_name, column_names, options)
-    @indexes[table_name] << IndexDefinition.new(table_name, index_name, (index_type == 'UNIQUE'), column_names, [], [])
-  end
 
-  unless instance_methods.include? :add_index_options
-    def add_index_options(table_name, column_name, options = {})
-      column_names = Array.wrap(column_name)
-      index_name   = index_name(table_name, :column => column_names)
+    index, index_type, ignore = add_index_options(table_name, column_names, **options)
 
-      if Hash === options # legacy support, since this param was a string
-        index_type = options[:unique] ? "UNIQUE" : ""
-        index_name = options[:name].to_s if options.key?(:name)
-      else
-        index_type = options
-      end
-
-      if index_name.length > index_name_length
-        raise ArgumentError, "Index name '#{index_name}' on table '#{table_name}' is too long; the limit is #{index_name_length} characters"
-      end
-      if index_name_exists?(table_name, index_name, false)
-        raise ArgumentError, "Index name '#{index_name}' on table '#{table_name}' already exists"
-      end
-      index_columns = quoted_columns_for_index(column_names, options).join(", ")
-
-      [index_name, index_type, index_columns]
+    if index.is_a?(ActiveRecord::ConnectionAdapters::IndexDefinition)
+      @indexes[table_name] << index
+    else
+      # Rails < 6.1
+      @indexes[table_name] << IndexDefinition.new(table_name, index, (index_type == 'UNIQUE'), column_names, [], [])
     end
   end
 
-  unless instance_methods.include? :index_name_exists?
-    def index_name_exists?(table_name, index_name, default)
-      return default unless respond_to?(:indexes)
-      index_name = index_name.to_s
-      indexes(table_name).detect { |i| i.name == index_name }
+  # Rails 6.1+
+  if ActiveRecord::VERSION::MAJOR >= 7 || (ActiveRecord::VERSION::MAJOR >= 6 and ActiveRecord::VERSION::MINOR > 0)
+    def remove_index(table_name, column_name = nil, **options )
+      index_name = index_name_for_remove(table_name, column_name, options)
+      index = @indexes[table_name].reject! { |index| index.name == index_name }
+    end
+  else
+    def remove_index(table_name,  options = {} )
+      index_name = index_name_for_remove(table_name, options)
+      index = @indexes[table_name].reject! { |index| index.name == index_name }
     end
   end
 
@@ -150,7 +145,7 @@ class ActiveRecord::ConnectionAdapters::NullDBAdapter < ActiveRecord::Connection
 
     if table = @tables[table_name]
       table.columns.map do |col_def|
-        col_args = new_column_arguments(col_def)
+        col_args = default_column_arguments(col_def)
         ActiveRecord::ConnectionAdapters::NullDBAdapter::Column.new(*col_args)
       end
     else
@@ -225,12 +220,51 @@ class ActiveRecord::ConnectionAdapters::NullDBAdapter < ActiveRecord::Connection
   end
 
   def primary_key(table_name)
-    columns(table_name).detect { |col| col.sql_type == :primary_key }.try(:name)
+    columns(table_name).detect { |col| col.type == :primary_key }.try(:name)
+  end
+
+  def add_column(table_name, column_name, type, **options)
+    super
+
+    table_meta = @tables[table_name.to_s]
+    return unless table_meta
+
+    table_meta.column column_name, type, **options
+  end
+
+  def change_column(table_name, column_name, type, options = {})
+    table_meta = @tables[table_name.to_s]
+    column = table_meta.columns.find { |column| column.name == column_name.to_s }
+    return unless column
+
+    column.type = type
+    column.options = options if options
+  end
+
+  def rename_column(table_name, column_name, new_column_name)
+    table_meta = @tables[table_name.to_s]
+    column = table_meta.columns.find { |column| column.name == column_name.to_s }
+    return unless column
+
+    column.name = new_column_name
+  end
+
+  def change_column_default(table_name, column_name, default_or_changes)
+    table_meta = @tables[table_name.to_s]
+    column = table_meta.columns.find { |column| column.name == column_name.to_s }
+
+    return unless column
+
+    if default_or_changes.kind_of? Hash
+      column.default = default_or_changes[:to]
+    else
+      column.default = default_or_changes
+    end
   end
 
   protected
 
-  def select(statement, name = nil, binds = [])
+  def select(statement, name = nil, binds = [], prepare: nil, async: nil)
     EmptyResult.new.tap do |r|
       r.bind_column_meta(columns_for(name))
       self.execution_log << Statement.new(entry_point, statement)
@@ -279,80 +313,43 @@ class ActiveRecord::ConnectionAdapters::NullDBAdapter < ActiveRecord::Connection
 
   def new_table_definition(adapter = nil, table_name = nil, is_temporary = nil, options = {})
     case ::ActiveRecord::VERSION::MAJOR
-    when 6
+    when 6, 7
       TableDefinition.new(self, table_name, temporary: is_temporary, options: options.except(:id))
     when 5
       TableDefinition.new(table_name, is_temporary, options.except(:id), nil)
-    when 4
-      TableDefinition.new(native_database_types, table_name, is_temporary, options)
-    when 2,3
-      TableDefinition.new(adapter)
     else
       raise "Unsupported ActiveRecord version #{::ActiveRecord::VERSION::STRING}"
     end
   end
 
-  def new_column_arguments(col_def)
-    args_with_optional_cast_type(col_def)
-  end
-
-  def args_with_optional_cast_type(col_def)
-    default_column_arguments(col_def).tap do |args|
-      if defined?(ActiveRecord::ConnectionAdapters::SqlTypeMetadata)
-        meta = ActiveRecord::ConnectionAdapters::SqlTypeMetadata.new(sql_type: col_def.type)
-        args.insert(2, meta_with_limit!(meta, col_def))
-      elsif initialize_column_with_cast_type?
-        args.insert(2, meta_with_limit!(lookup_cast_type(col_def.type), col_def))
-      else
-        args[2] = args[2].to_s + "(#{col_def.limit})" if col_def.limit
-      end
-    end
-  end
-
-  def meta_with_limit!(meta, col_def)
-    meta.instance_variable_set('@limit', col_def.limit)
-    meta
-  end
-
   def default_column_arguments(col_def)
-    if ActiveRecord::VERSION::MAJOR >= 5
-      [
-        col_def.name.to_s,
-        col_def.default,
-        col_def.null.nil? || col_def.null # cast  [false, nil, true] => [false, true, true], other adapters default to null=true 
-      ]
-    else
-      [
-        col_def.name.to_s,
-        col_def.default,
-        col_def.type,
-        col_def.null.nil? || col_def.null # cast  [false, nil, true] => [false, true, true], other adapters default to null=true 
-      ]
-    end
+    [
+      col_def.name.to_s,
+      col_def.default.present? ? col_def.default.to_s : nil,
+      sql_type_definition(col_def),
+      col_def.null.nil? || col_def.null
+    ]
   end
 
-  def initialize_column_with_cast_type?
-    ::ActiveRecord::VERSION::MAJOR == 4 && ::ActiveRecord::VERSION::MINOR >= 2
+  def sql_type_definition(col_def)
+    ActiveRecord::ConnectionAdapters::SqlTypeMetadata.new(
+      type: col_def.type,
+      sql_type: col_def.type.to_s,
+      limit: col_def.limit
+    )
   end
 
   def initialize_args
-    return [nil, @logger, @config] if ActiveRecord::VERSION::MAJOR > 3
-    [nil, @logger]
+    [nil, @logger, @config]
   end
 
-  # 4.2 introduced ActiveRecord::Type
-  # https://github.com/rails/rails/tree/4-2-stable/activerecord/lib/active_record
   def register_types
-    if ActiveRecord::VERSION::MAJOR < 5
-      type_map.register_type(:primary_key, ActiveRecord::Type::Integer.new)
-    else      
-      require 'active_model/type'
-      ActiveRecord::Type.register(
-        :primary_key,
-        ActiveModel::Type::Integer,
-        adapter: adapter_name,
-        override: true
-      )
-    end
+    require 'active_model/type'
+    ActiveRecord::Type.register(
+      :primary_key,
+      ActiveModel::Type::Integer,
+      adapter: adapter_name,
+      override: true
+    )
   end
 end
