@@ -28,14 +28,4 @@ class ActiveRecord::Base
   def self.nulldb_connection(config)
     ActiveRecord::ConnectionAdapters::NullDBAdapter.new(config)
   end
-end
-
-
-module ActiveRecord
-  # Just make sure you have the latest version of your schema
-  class Schema < Migration
-    def self.define(info={}, &block)
-      instance_eval(&block)
-    end
-  end
-end
+end
\ No newline at end of file
